import java.awt.Point;

//
// 変数の準備
//

Point kuma = new Point();
Point dragon = new Point();

PImage image_kuma;
PImage image_dragon;
PImage image_background;

String game_status = "playing";
String jump_status = "running";
int height = 0;
long t_jump = 0;      // ジャンプを開始した時間
long t_dragon = 0;    // 敵が出現した時間

//
// 起動時に呼ばれる組み込み関数
//
void setup() {
  size(500, 500);

  image_background =   loadImage("background.png");
  image_kuma=   loadImage("kuma.png");
  image_dragon=   loadImage("dragon.png");

  game_start();
}

void game_start() {
  game_status = "playing";

  t_dragon = System.currentTimeMillis();
  height = 0;
  jump_status = "running";
}

//
// 毎フレーム呼ばれる組み込み関数
//
void draw() {
  if (game_status.equals("playing")) 
    update_character_positions();

  image(image_background, 0, 0);
  image(image_kuma, kuma.x, kuma.y);
  image(image_dragon, dragon.x, dragon.y);

  if (game_status.equals("game_over")) {
    draw_message("GAME OVER!");
  }
}


void update_character_positions() {
  //
  // 自分の移動　　上下移動
  //
  if (jump_status.equals("running"))
    height = 0;
  else {
    double jt = (int)(System.currentTimeMillis()-t_jump)/1000.0;
    height = (int)(300*jt - 300*jt*jt);

    if (height < 0) {
      // 着地した
      height = 0;
      jump_status = "running";

      // jump中に押したら着地時にすぐジャンプ
      if (pressed) {
        t_jump = System.currentTimeMillis();
        jump_status = "jumping";
      }
    }
  }
  kuma.x = 50;
  kuma.y = 350-height;

  //
  // 敵の移動　　右は左へ
  //
  int dt = (int)(System.currentTimeMillis()-t_dragon); 
  dragon.x = (int)(450- dt/1000.0*200);
  dragon.y = 350;
  if (dragon.x < -50) {
    t_dragon = System.currentTimeMillis();  //左端に来たら右端に戻す
  }

  //
  // 衝突判定
  //
  double d = Math.sqrt((kuma.x-dragon.x)*(kuma.x-dragon.x) + (kuma.y-dragon.y)*(kuma.y-dragon.y));
  if (d < 40) 
    game_status = "game_over";
}


//
// テキストの表示
//
void draw_message(String message) {
  textFont (createFont("MS Gothic", 80, true));  // 文字のフォント
  fill(0);                                     // 文字の色
  textSize(24);                                // 文字のサイズ
  text(message, 150, 240);
}



//
// マウス入力の処理
//
boolean pressed = false;  // jump中に押したら着地時にすぐジャンプするためのフラグ

void mousePressed() {
    pressed = true;
    if (jump_status.equals("running")) {
      jump_status = "jumping";
      t_jump = System.currentTimeMillis();
    }
}
void mouseReleased() {
  pressed = false;
  if (game_status.equals("game_over")) {
    game_start();
  }
}